import 'dart:async';
import 'dart:convert' as convert;
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:witsystem_core/error/entity/WitsystemError.dart';
import 'package:witsystem_core/error/handle/AutoErrorHandle.dart';
import 'package:witsystem_core/pages/witDialog/DialogCenterHintWidget.dart';
import 'package:witsystem_core/permission/WitsPermission.dart';
import 'package:witsystem_device_manage/assets/icons/WitsIcon.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_manage/pages/home/add/AddCustomWidget.dart';
import 'package:witsystem_device_manage/pages/home/slock/info/DeviceInfo.dart';
import 'package:witsystem_device_plugin/device/AddDevice.dart';
import 'package:witsystem_device_plugin/device/DeviceManage.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';

class AddDeviceWidget extends StatefulWidget {
  const AddDeviceWidget({Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return AddDeviceState();
  }
}

class AddDeviceState extends State<AddDeviceWidget> {
  bool isAdd = false;
  StreamSubscription? listen;
  final List<Map> _record = [];

  @override
  void initState() {
    super.initState();
    _scan();
  }

  ///扫描设备
  _scan() {
    if (isAdd) return;
    isAdd = true;
    _record.clear();
    _record.add({"text": S.current.device_startScan, "color": Colors.blue});
    setState(() {});
    Stream<Map<String, dynamic>> scanAddBleDevice =
        AddDevice().scanAddBleDevice();
    listen = scanAddBleDevice.listen((event) {
      debugPrint("扫描到的设备$event");
      _stop();
      _record.add({
        "text": S.current.device_scanToDevice(event["deviceId"]),
        "color": Colors.blue
      });
      setState(() {});
      Future.delayed(const Duration(milliseconds: 100), () {
        _connect(event["deviceId"]);
      });
    })
      ..onError((err) async {
        isAdd = false;
        WitsystemError witsystemError = AutoErrorHandle.auto(err);
        _record.add({"text": witsystemError.error, "color": Colors.red});
        _record
            .add({"text": S.current.device_addRetryHint, "color": Colors.red});
        setState(() {});
        debugPrint("错误信息$err");
        if (witsystemError.code == 110003) {
          //缺少权限去申请权限
          if (!await WitsPermission.blePermission()) {
            _requestPermission();
          }
        }
      });
  }

  //申请权限失败提示
  _requestPermission() {
    DialogCenterHint().show(
      context: context,
      title: S.current.device_permissionFailedTitle,
      content: S.current.device_permissionFailedHint,
      outSideDismissible: true,
      affirmText: S.of(context).device_retry,
      affirmTextColor: Colors.green,
      onPressed: (bool value) async {
        if (!value) {
          Navigator.of(context).pop();
          return;
        }
        if (!await WitsPermission.blePermission()) {
          _requestPermission();
        }
      },
    );
  }

  //停止扫描
  _stop() {
    AddDevice().stopAddBleDevice(listen);
  }

  //开始连接设备
  _connect(String deviceId) async {
    _record.add(
        {"text": S.current.device_addDeviceConnecting, "color": Colors.blue});
    _record
        .add({"text": S.current.device_interactiveData, "color": Colors.blue});
    setState(() {});
    try {
      List list = await DeviceManage().deviceList();
      await AddDevice().addBleDevice(
          deviceId, S.current.device_intelligentDevice(list.length),waitSettingsState:(String deviceId){
        _record
            .add({"text": S.current.device_clickSetupKey, "color": Colors.blue});
        setState(() {});
      });
      _record.add({"text": S.current.device_addSuccess, "color": Colors.blue});
      isAdd = false;
      setState(() {});
      EasyLoading.showToast(S.current.device_addSuccess);
      Future.delayed(const Duration(milliseconds: 100), () {
        Navigator.of(context).pop({"refresh": true});
        Navigator.of(context).push(MaterialPageRoute(builder: (context) {
          return DeviceInfoWidget(deviceId: deviceId);
        }));
      });
    } on PlatformException catch (e) {
      debugPrint("连接失败$e");
      WitsystemError witsystemError = AutoErrorHandle.auto(e);
      String errorMsg = witsystemError.error;
      if (e.code == "660004") {
        var decode = convert.json.decode(e.message!);
        errorMsg = errorMsg +
            ", '${S.current.device_deviceExistUserHint(decode["userId"])}'";
      }
      _record.add(
          {"text": S.current.device_addFail(errorMsg), "color": Colors.red});
      _record.add({"text": S.current.device_addRetryHint, "color": Colors.red});
      isAdd = false;
      setState(() {});
    }
  }

  @override
  void dispose() {
    super.dispose();
    _stop();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(S.of(context).device_add),
      ),
      body: Column(
        mainAxisAlignment: MainAxisAlignment.spaceAround,
        children: [
          SizedBox(
            width: 300,
            height: 300,
            child: Stack(alignment: Alignment.center, children: [
              AddCustomWidget(
                isExe: isAdd,
                color: Colors.black,
                dotNumber: 100,
              ),
              Positioned(
                  top: 20,
                  child: Container(
                      color: Colors.white,
                      child: const Icon(WitsIcon.yunduan, size: 45))),
              Positioned(
                  left: 25,
                  child: Container(
                      color: Colors.white,
                      child: const Icon(WitsIcon.shoujihao, size: 45))),
              Positioned(
                  right: 25,
                  child: Container(
                      color: Colors.white,
                      child: const Icon(WitsIcon.shebei, size: 45))),
              AnimatedCrossFade(
                duration: const Duration(milliseconds: 300),
                crossFadeState: !isAdd
                    ? CrossFadeState.showFirst
                    : CrossFadeState.showSecond,
                firstChild: MaterialButton(
                    onPressed: _scan, child: Text(S.of(context).device_retry)),
                secondChild: Text(S.of(context).device_adding),
              )
            ]),
          ),
          Expanded(
              child: Container(
            width: double.infinity,
            //color: Colors.blue,
            margin: const EdgeInsets.all(10),
            child: ListView.builder(
              itemCount: _record.length,
              itemBuilder: (context, index) {
                return Padding(
                  padding: const EdgeInsets.fromLTRB(0, 0, 0, 10),
                  child: Text(
                    _record[index]["text"],
                    style: TextStyle(color: _record[index]["color"]),
                  ),
                );
              },
            ),
          )),
        ],
      ),
    );
  }
}
