import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:witsystem_core/error/NetError.dart';
import 'package:witsystem_core/pages/sheet/WitsystemBottomSheet.dart';
import 'package:witsystem_core/pages/witDialog/DialogCenterHintWidget.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_plugin/device/DeviceManage.dart';

///绑定网关页面

class BindRelayWidget extends StatefulWidget {
  final String deviceId;

  const BindRelayWidget({Key? key, required this.deviceId}) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _BindRelayState();
  }
}

class _BindRelayState extends State<BindRelayWidget> {
  String deviceName = "";

  Map? deviceInfo; //当前设备的信息

  Map? bindRelayInfo; //绑定的网关信息

  @override
  void initState() {
    super.initState();
    _getDeviceInfo();
  }

  ///获得设备信息
  _getDeviceInfo() async {
    deviceInfo = await DeviceManage().device(widget.deviceId);
    deviceName = deviceInfo!["deviceName"];
    if (deviceInfo!["device"]["superDeviceId"] != null) {
      bindRelayInfo = await DeviceManage().device(deviceInfo!["device"]
          ["superDeviceId"]); //获得中继信息 获得到代表自己有中继权限，获取不到代表没有中继权限
    } else {
      bindRelayInfo = null;
    }
    setState(() {});
  }

  ///返回可以选择的网关信息列表
  Future<List> _getRelayList() async {
    List list = await DeviceManage().deviceList();
    list.removeWhere((element) => element["device"]["type"] != "Relay"); //筛选出网关
    return list
        .map((e) => {"deviceName": e["deviceName"], "deviceId": e["deviceId"]})
        .toList()
      ..add({"deviceName": S.current.device_cancel}); //只要网关的名称
  }

  ///弹出底部网关选择列表
  _relaySelectList() async {
    List list = await _getRelayList();
    WitsystemBottomSheet().show(
      context: context,
      showKey: "deviceName",
      sheetItem: list,
      onTap: (index) {
        if (list[index]["deviceId"] == null) return;
        bindRelayInfo = list[index];
        setState(() {});
      },
    );
  }

  ///绑定网关
  bindRelay() async {
    try {
      EasyLoading.show(
          status: S.current.device_loading,
          maskType: EasyLoadingMaskType.clear);
      await DeviceManage()
          .bindRelay(bindRelayInfo!["deviceId"], widget.deviceId);
      EasyLoading.showSuccess(S.current.device_success);
      _getDeviceInfo();
    } on PlatformException catch (e) {
      EasyLoading.dismiss();
      EasyLoading.showError(NetError.error(int.parse(e.code)).error);
    }
  }

  ///点击已经绑定中继
  unbindRelay() {
    DialogCenterHint().show(
      context: context,
      title: S.current.device_unbindRelay,
      content: S.current.device_unbindRelayHint(deviceName),
      outSideDismissible: true,
      affirmText: S.of(context).device_unbind,
      affirmTextColor: Colors.red,
      onPressed: (bool value) {
        if (value) affirmUnbindRelay();
      },
    );
  }

  ///确认解绑
  affirmUnbindRelay() async {
    try {
      EasyLoading.show(
          status: S.current.device_unbinding,
          maskType: EasyLoadingMaskType.clear);
      await DeviceManage().unbindRelay(widget.deviceId);
      EasyLoading.showSuccess(S.current.device_success);
      _getDeviceInfo();
    } on PlatformException catch (e) {
      EasyLoading.dismiss();
      EasyLoading.showError(NetError.error(int.parse(e.code)).error);
    }
  }

  @override
  void dispose() {
    super.dispose();
    EasyLoading.dismiss();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar:
          AppBar(title: Text(S.of(context).device_bindRelay + "($deviceName)")),
      body: ListView(children: [
        ListTile(
            contentPadding: const EdgeInsets.fromLTRB(15, 5, 15, 5),
            title: Text(S.of(context).device_deviceName),
            trailing: Text(deviceName,
                style: const TextStyle(color: Colors.black54, fontSize: 14))),
        const Divider(height: 1),
        (deviceInfo == null || deviceInfo!["device"]["superDeviceId"] == null)
            ? ListTile(
                onTap: _relaySelectList,
                contentPadding: const EdgeInsets.fromLTRB(15, 5, 15, 5),
                title: Text(S.of(context).device_relayName),
                trailing: Row(
                  mainAxisSize: MainAxisSize.min,
                  children: [
                    Text(
                        bindRelayInfo == null
                            ? S.current.device_bindRelayHint
                            : bindRelayInfo!["deviceName"],
                        style: const TextStyle(
                            color: Colors.black54, fontSize: 14)),
                    const Icon(Icons.keyboard_arrow_right)
                  ],
                ))
            : ListTile(
                onTap: unbindRelay,
                contentPadding: const EdgeInsets.fromLTRB(15, 5, 15, 5),
                title: Text(S.of(context).device_relayName),
                trailing: Row(
                  mainAxisSize: MainAxisSize.min,
                  children: [
                    Text(
                        S.current.device_alreadyBindRelay +
                            (bindRelayInfo != null
                                ? "(${bindRelayInfo!["deviceName"]})"
                                : ""),
                        style: const TextStyle(
                            color: Colors.black54, fontSize: 14)),
                    const Icon(Icons.keyboard_arrow_right)
                  ],
                )),
        const Divider(height: 1),
        AnimatedSwitcher(
          duration: const Duration(milliseconds: 200),
          transitionBuilder: (Widget child, Animation<double> animation) {
            return ScaleTransition(child: child, scale: animation); //进行自定义的切换动画
          },
          child: ((deviceInfo == null ||
                      deviceInfo!["device"]["superDeviceId"] == null) &&
                  bindRelayInfo != null) //判断没有绑定并且已经选择了需要绑定额网关显示button
              ? Padding(
                  padding: const EdgeInsets.fromLTRB(10, 50, 10, 30),
                  child: MaterialButton(
                    onPressed: bindRelay,
                    shape: RoundedRectangleBorder(
                      side: BorderSide(
                        color: Theme.of(context).primaryColorLight,
                        width: 1,
                      ),
                      borderRadius: const BorderRadius.all(
                        Radius.circular(8),
                      ),
                    ),
                    color: Theme.of(context).primaryColorLight,
                    minWidth: double.infinity,
                    padding: const EdgeInsets.fromLTRB(10, 20, 10, 20),
                    child: Text(S.of(context).device_bindRelay,
                        style: const TextStyle(color: Colors.white)),
                  ),
                )
              : const SizedBox(height: 0, width: 0),
        )
      ]),
    );
  }
}
