import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:witsystem_core/config/WitsystemConfig.dart';
import 'package:witsystem_core/mqtt/MqttException.dart';
import 'package:witsystem_core/mqtt/MqttTool.dart';
import 'package:witsystem_core/mqtt/WitsMqtt.dart';
import 'package:witsystem_core/pages/code/WitsQrIamge.dart';
import 'package:witsystem_device_manage/assets/icons/WitsIcon.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'dart:convert' as convert;

import 'package:witsystem_device_plugin/device/DeviceManage.dart';

///当面分享二维码信息

class ShortShareInfoWidget extends StatefulWidget {
  final String deviceName;
  final String deviceId;
  final String? startTime;
  final String? endTime;
  final String authType;
  final String repeatType;

  const ShortShareInfoWidget(
      {Key? key,
      required this.deviceName,
      required this.deviceId,
      this.startTime,
      this.endTime,
      required this.authType,
      required this.repeatType})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _ShortShareInfoState();
  }
}

class _ShortShareInfoState extends State<ShortShareInfoWidget> {

  List scanUserList = []; //扫描用户列表

  Map qrCodeInfo = {};

  bool isExpire = false; //二维码是否过期

  Timer? timer; //定时器

  String? userUuid;

  @override
  void initState() {
    super.initState();
    _createdQrCodeInfo();
    _qrCodeTimeout();
    _mqttConnect();
  }

  ///获得设备信息
  Future<String> _getDeviceUserUuid() async {
    if (userUuid != null) return userUuid!;
    Map<String, dynamic>? deviceInfo =
        await DeviceManage().device(widget.deviceId);
    userUuid = deviceInfo!["userUuid"];
    return userUuid!;
  }

  ///生成qrCodeInfo
  _createdQrCodeInfo() async {
    qrCodeInfo["deviceId"] = widget.deviceId;
    qrCodeInfo["authType"] = widget.authType;
    if (widget.startTime != null) qrCodeInfo["startDate"] = widget.startTime;
    if (widget.endTime != null) qrCodeInfo["endDate"] = widget.endTime;
    qrCodeInfo["repeatType"] = widget.repeatType;
    qrCodeInfo["codeUid"] = DateTime.now().millisecondsSinceEpoch;
    qrCodeInfo["userUuid"] = await _getDeviceUserUuid();
    setState(() {});
  }

  ///重新生成二维码
  _regenerateQrCodeInfo() async {
    isExpire = false;
    await _createdQrCodeInfo();
    if (timer != null) timer!.cancel();
    _qrCodeTimeout();
  }

  ///启动连接mqtt
  _mqttConnect() async {
    try {
      EasyLoading.show(status: S.current.device_loading);
      WitsMqtt().disconnected();
      await WitsMqtt().connect();
      WitsMqtt().addObserver(_call);
      EasyLoading.dismiss();
    } on MqttException catch (e) {
      isExpire = true;
      setState(() {});
      EasyLoading.dismiss();
      EasyLoading.showError(e.message!);
    }
  }

  ///mqtt观察者
  _call(mqttData) async {
    var decode = convert.json.decode(mqttData.data);
    debugPrint("观察者收到的消息$decode");
    //判断不是扫描二维码的信息直接拒绝
    if (decode["type"] != 1005) {
      return;
    }

    //判断是否已经失效
    if (isExpire) {
      EasyLoading.showToast(S.of(context).device_expireQrCode);
      return;
    }

    var parse = convert.json.decode(decode["msg"]);
    //判断是否是当前二维码
    if (parse["codeUid"].toString() != qrCodeInfo["codeUid"].toString()) {
      EasyLoading.showToast(S.of(context).device_wrongQrCode);
      return;
    }

    //判断是否是当前设备
    if (parse["deviceId"] != widget.deviceId) {
      EasyLoading.showToast(S.of(context).device_wrongDeviceId);
      return;
    }
    await DeviceManage().sendDevice(
        parse["userId"],
        widget.deviceId,
        widget.startTime==null ? "" : widget.startTime!,
        widget.endTime==null ? "" : widget.endTime!,
        widget.authType,
        widget.repeatType);
    scanUserList.add(parse["userId"]);
    String createdReplyData = MqttTools.createdReplyData("00", mqttData);
    WitsMqtt().publishMessage(mqttData.topic, createdReplyData);
    setState(() {});
  }

  ///定时器
  _qrCodeTimeout() {
    timer = Timer(const Duration(seconds: 300), () {
      setState(() {
        isExpire = true;
      });
    });
  }

  @override
  void dispose() {
    super.dispose();
    WitsMqtt().revomeObserver(_call);
    WitsMqtt().disconnected(); //临时调用断开，是否需要断开后面在决定
    if (timer != null) timer!.cancel();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(S.of(context).device_shortShareQrCode+"(${widget.deviceName})"),
      ),
      body: Container(
        padding: const EdgeInsets.fromLTRB(10, 20, 10, 0),
        width: double.infinity,
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            Stack(
              alignment: AlignmentDirectional.center,
              children: [
                WitsQrIamge(
                  data: WitsystemConfig().deviceQrcodeUrl(
                      deviceId: widget.deviceId,
                      action:
                          "scanQrCoderSendKey&qrCodeInfo=${convert.json.encode(qrCodeInfo)}"),
                  size: 240.0,
                ),
                Positioned(
                    top: 0,
                    child: GestureDetector(
                        onTap: _regenerateQrCodeInfo,
                        child: Container(
                          alignment: Alignment.center,
                          color: Colors.black54,
                          width: isExpire ? 240 : 0,
                          height: isExpire ? 240 : 0,
                          child:
                              Column(mainAxisSize: MainAxisSize.min, children: [
                            const Icon(WitsIcon.shuaxin,
                                color: Colors.white, size: 40),
                            const Padding(padding: EdgeInsets.all(5)),
                            Text(S.of(context).device_qrcodeExpired,
                                style: const TextStyle(color: Colors.white))
                          ]),
                        )))
              ],
            ),
            Padding(
              padding: const EdgeInsets.all(10),
              child: Text(S.of(context).device_shortShareHint),
            ),
            Container(
                width: double.infinity,
                padding: const EdgeInsets.fromLTRB(10, 15, 0, 15),
                decoration: BoxDecoration(
                    border: Border(
                        top: const BorderSide(width: 1, color: Colors.black12),
                        bottom:
                            const BorderSide(width: 1, color: Colors.black12),
                        left: BorderSide(
                            width: 5,
                            color: Theme.of(context).primaryColorLight))),
                child: Text(
                  S.current.device_alreadyPermissions,
                  style: const TextStyle(fontWeight: FontWeight.w600),
                )),
            Expanded(
                child: scanUserList.isEmpty
                    ? Container(
                        alignment: Alignment.center,
                        child:
                            Column(mainAxisSize: MainAxisSize.min, children: [
                          const Icon(WitsIcon.meiyoushuju, size: 50),
                          const Padding(padding: EdgeInsets.all(5)),
                          Text(S.of(context).device_notUserScan),
                        ]),
                      )
                    : ListView.separated(
                        itemCount: scanUserList.length,
                        itemBuilder: (context, index) {
                          return ListTile(
                              minLeadingWidth: 0,
                              leading: const Icon(WitsIcon.saoma),
                              title: Text(
                                  S.of(context).device_user + index.toString()),
                              subtitle: Text(
                                scanUserList[index],
                                style: TextStyle(
                                    color: Theme.of(context).primaryColorLight),
                              ));
                        },
                        separatorBuilder: (BuildContext context, int index) {
                          return const Divider(height: 1);
                        },
                      ))
          ],
        ),
      ),
    );
  }
}
