import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:witsystem_core/error/entity/WitsystemError.dart';
import 'package:witsystem_core/error/handle/AutoErrorHandle.dart';
import 'package:witsystem_core/pages/sheet/WitsystemBottomSheet.dart';
import 'package:witsystem_core/util/WitsRegExp.dart';
import 'package:witsystem_device_manage/assets/icons/WitsIcon.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_plugin/command/CardType.dart';
import 'package:witsystem_device_plugin/device/BleDevice.dart';
import 'package:witsystem_device_plugin/device/DeviceManage.dart';
import 'package:witsystem_device_plugin/enity/DeviceSoftwareInfo.dart';

///添加卡
class AddCardWidget extends StatefulWidget {
  final String deviceId;
  final String deviceName;

  const AddCardWidget(
      {Key? key, required this.deviceId, required this.deviceName})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _AddCardState();
  }
}

class _AddCardState extends State<AddCardWidget> {
  bool _isUpdateList = false; //上一页是否需要更新密码列表
  final _authType = TextEditingController();
  final _cardName = TextEditingController();
  final _userId = TextEditingController();

  final Map<CardType, dynamic> _supportCardType = {
    CardType.ADMIN_CARD: S.current.device_adminCard,
    CardType.USER_CARD: S.current.device_userCard,
  };

  @override
  void initState() {
    super.initState();
    _filterSupportPasswordType();
  }

  ///筛选设备支持哪些卡
  _filterSupportPasswordType() async {
    DeviceSoftwareInfo? softwareInfo =
        await DeviceManage().getDeviceSoftwareFunction(widget.deviceId);
    _supportCardType
        .removeWhere((key, value) => !softwareInfo!.cardType.contains(key));
  }

  ///选择权限
  _selectAuth() {
    Future.delayed(const Duration(milliseconds: 160), () {
      WitsystemBottomSheet().show(
        context: context,
        sheetItem: _supportCardType.values.toList()
          ..add(S.current.device_cancel),
        onTap: (index) {
          if (index == _supportCardType.length) {
            _authType.text = "";
          } else {
            _authType.text = _supportCardType.values.toList()[index];
          }
        },
      );
    });
  }

  ///给卡名称添加动画和判断是否显示
  Widget _widgetCardName({required Widget child}) {
    return AnimatedSwitcher(
      duration: const Duration(milliseconds: 200),
      transitionBuilder: (Widget child, Animation<double> animation) {
        return ScaleTransition(child: child, scale: animation); //进行自定义的切换动画
      },
      child: _authType.value.text.isNotEmpty
          ? child
          : const SizedBox(height: 0, width: 0),
    );
  }

  ///给是使用者账号添加动画和判断是否显示
  Widget _widgetUserId({required Widget child}) {
    return AnimatedSwitcher(
      duration: const Duration(milliseconds: 200),
      transitionBuilder: (Widget child, Animation<double> animation) {
        return ScaleTransition(child: child, scale: animation); //进行自定义的切换动画
      },
      child: _authType.value.text.isNotEmpty && _cardName.value.text.isNotEmpty
          ? child
          : const SizedBox(height: 0, width: 0),
    );
  }

  ///点击添加卡
  _addCard() async {
    if (_authType.text.isEmpty) {
      EasyLoading.showToast(S.current.device_selectAuthType);
      return;
    }
    if (_cardName.text.isEmpty) {
      EasyLoading.showToast(S.current.device_pwdNameHint);
      return;
    }

    if (_userId.text.isNotEmpty &&
        !WitsRegExp.isEmail(_userId.text) &&
        !WitsRegExp.isChinaPhoneLegal(_userId.text)) {
      EasyLoading.showToast(S.current.device_userErrorHint);
      return;
    }
    EasyLoading.show(status: S.current.device_loading,maskType: EasyLoadingMaskType.clear);
    try {
      await BleDevice().addCard(
          widget.deviceId, __authTypeValueToInt(), _cardName.text,
          userId: _userId.text.isEmpty ? null : _userId.text, swingCard: () {
        EasyLoading.dismiss();
        EasyLoading.show(
            status: S.current.device_swingCard,
            maskType: EasyLoadingMaskType.clear);
      });
      _isUpdateList = true;
      EasyLoading.dismiss();
      EasyLoading.showSuccess(S.current.device_success);
      _initController();
    } on PlatformException catch (e) {
      EasyLoading.dismiss();
      WitsystemError error = AutoErrorHandle.auto(e);
      EasyLoading.showToast(error.error);
    }
  }

  ///_authType 的值读取权限标记
  CardType __authTypeValueToInt() {
    return _supportCardType.keys.toList()[
        _supportCardType.values.toList().indexOf(_authType.value.text)];
  }

  ///清空控制器
  _initController() {
    _authType.text = "";
    _cardName.text = "";
    _userId.text = "";
    setState(() {});
  }

  @override
  void dispose() {
    super.dispose();
    _authType.dispose();
    _cardName.dispose();
    _userId.dispose();
    EasyLoading.dismiss();
    BleDevice().cancelAddCard(widget.deviceId);
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () {
        Navigator.pop(context, _isUpdateList);
        return Future.value(false);
      },
      child: Scaffold(
          appBar: AppBar(
              title: Text(
                  S.of(context).device_addCard + "(${widget.deviceName})",
                  style: const TextStyle(fontSize: 16))),
          body: GestureDetector(
              onTap: () {
                FocusScope.of(context).requestFocus(FocusNode()); // 触摸收起键盘
              },
              child: Container(
                margin: const EdgeInsets.fromLTRB(10, 0, 10, 10),
                child: ListView(
                  children: [
                    Row(
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [
                        Container(
                          height: 120,
                          width: 120,
                          margin: const EdgeInsets.fromLTRB(0, 20, 0, 20),
                          decoration: BoxDecoration(
                              border: Border.all(
                                  color: Theme.of(context).primaryColorLight),
                              borderRadius:
                                  const BorderRadius.all(Radius.circular(60))),
                          child: Icon(WitsIcon.mensuo,
                              size: 50,
                              color: Theme.of(context).primaryColorLight),
                        ),
                      ],
                    ),
                    SizedBox(
                      height: 60,
                      child: TextField(
                        onTap: () {
                          FocusScope.of(context)
                              .requestFocus(FocusNode()); // 触摸收起键盘
                          _selectAuth();
                        },
                        enableInteractiveSelection: false,
                        controller: _authType,
                        style: const TextStyle(fontSize: 14),
                        decoration: InputDecoration(
                            prefixIcon: const Icon(Icons.key),
                            suffixIcon: const Icon(WitsIcon.xiala),
                            hintText: S.of(context).device_selectAuthType,
                            border: const OutlineInputBorder(
                                borderRadius:
                                    BorderRadius.all(Radius.circular(10)),
                                borderSide: BorderSide(color: Colors.black12))),
                      ),
                    ),
                    _widgetCardName(
                      child: Container(
                        margin: const EdgeInsets.fromLTRB(0, 10, 0, 10),
                        child: TextField(
                          onChanged: (v) {
                            setState(() {});
                          },
                          controller: _cardName,
                          enableInteractiveSelection: false,
                          style: const TextStyle(fontSize: 14),
                          decoration: InputDecoration(
                              hintText: S.of(context).device_cardHint,
                              prefixIcon: const Icon(WitsIcon.gaojian),
                              border: const OutlineInputBorder(
                                  borderRadius:
                                      BorderRadius.all(Radius.circular(10)),
                                  borderSide:
                                      BorderSide(color: Colors.black12))),
                        ),
                      ),
                    ),
                    _widgetUserId(
                      child: Container(
                        margin: const EdgeInsets.fromLTRB(0, 0, 0, 60),
                        child: TextField(
                          onChanged: (v) {
                            setState(() {});
                          },
                          keyboardType: TextInputType.emailAddress,
                          controller: _userId,
                          style: const TextStyle(fontSize: 14),
                          enableInteractiveSelection: false,
                          decoration: InputDecoration(
                              hintText: S.of(context).device_inputUserHint,
                              prefixIcon: const Icon(WitsIcon.yonghu, size: 20),
                              border: const OutlineInputBorder(
                                  borderRadius:
                                      BorderRadius.all(Radius.circular(10)),
                                  borderSide:
                                      BorderSide(color: Colors.black12))),
                        ),
                      ),
                    ),
                    _widgetUserId(
                        child: MaterialButton(
                      onPressed: _addCard,
                      child: Text(S.of(context).device_addCard,
                          style: const TextStyle(color: Colors.white)),
                      color: Colors.blue,
                      height: 60,
                      minWidth: double.infinity,
                    )),
                  ],
                ),
              ))),
    );
  }
}
