import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:witsystem_core/error/entity/WitsystemError.dart';
import 'package:witsystem_core/error/handle/AutoErrorHandle.dart';
import 'package:witsystem_core/pages/sheet/WitsystemBottomSheet.dart';
import 'package:witsystem_core/pages/witDialog/DialogCenterHintWidget.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_manage/pages/home/slock/card/AddCard.dart';
import 'package:witsystem_device_manage/pages/widget/FloatingActionButtonScaffold.dart';
import 'package:witsystem_device_manage/util/DeviceInfoAuth.dart';
import 'package:witsystem_device_plugin/device/BleDevice.dart';
import 'package:witsystem_device_plugin/device/DeviceManage.dart';

///门卡的列表
///需要进行时间国际化
class CardListWidget extends StatefulWidget {
  final String deviceId;

  const CardListWidget({Key? key, required this.deviceId}) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _CardListState();
  }
}

class _CardListState extends State<CardListWidget> {
  String deviceName = "";
  List colorList = const [
    Color(0xFF51B6FF),
    Color(0xFFFFAA63),
    Color(0xFFBD4FEF),
    Color(0xFFEA4C89)
  ];
  List list = [];
  Map freeze = {"text": S.current.device_freeze, "type": "freeze"};
  Map unfreeze = {"text": S.current.device_unfreeze, "type": "unfreeze"};
  final List<Map> _operateSheetItem = [
    {"text": S.current.device_delete, "type": "delete"},
    {},
    {"text": S.current.device_cancel, "type": "cancel"},
  ];

  @override
  void initState() {
    super.initState();
    initData();
  }

  ///初始化数据
  initData() async {
    Map<String, dynamic>? device = await DeviceManage().device(widget.deviceId);
    deviceName = device!["deviceName"];
    list.clear();
    if (device["device"]["devicesSlock"] == null) {
      setState(() {});
      return;
    }
    List deviceList = device["device"]["devicesSlock"];
    int length = deviceList.length;
    for (int i = length - 1; i >= 0; i--) {
      if (deviceList[i]["type"] == "ID_CARD") {
        deviceList[i]["dataType"] =
            DeviceInfoAuth.cardAuthToString(deviceList[i]["dataType"]);
        list.add(deviceList[i]);
      }
    }
    //print("获得的设备数据${list}");
    setState(() {});
  }

  ///刷新
  refurbishDevice() async {
    await DeviceManage().refurbishDevice(widget.deviceId);
    await initData();
  }

  ///点击卡片
  _cardOnTap(int cardIndex) {
    _operateSheetItem[1] = list[cardIndex]["freeze"] ? unfreeze : freeze;
    WitsystemBottomSheet().show(
      context: context,
      sheetItem: _operateSheetItem,
      showKey: "text",
      onTap: (index) {
        switch (_operateSheetItem[index]["type"]) {
          case "delete":
            _deleteCard(cardIndex);
            break;
          case "freeze":
          case "unfreeze":
            _freezeCard(cardIndex);
            break;
        }
      },
    );
  }

  ///删除
  _deleteCard(int index) {
    DialogCenterHint().show(
      context: context,
      title: S.current.device_deleteCardHintTitle,
      content: S.current.device_deleteCardHint(list[index]["name"]),
      outSideDismissible: true,
      affirmText: S.of(context).device_delete,
      affirmTextColor: Colors.red,
      onPressed: (bool value) {
        if (value) _affirmDelete(index);
      },
    );
  }

  ///确认删除
  _affirmDelete(int index) async {
    try {
      EasyLoading.show(status: S.current.device_loading);
      await BleDevice().deleteCard(widget.deviceId, list[index]["uid"]);
      EasyLoading.dismiss();
      EasyLoading.showSuccess(S.current.device_success);
      initData();
    } on PlatformException catch (e) {
      print(e);
      EasyLoading.dismiss();
      WitsystemError error = AutoErrorHandle.auto(e);
      EasyLoading.showToast(error.error + "${e.message}");
    }
  }

  ///冻结或解冻
  _freezeCard(int index) {
    DialogCenterHint().show(
      context: context,
      title: list[index]["freeze"]
          ? S.current.device_unfreezeCardHintTitle
          : S.current.device_freezeCardHintTitle,
      content: list[index]["freeze"]
          ? S.current.device_unfreezeCardHint(list[index]["name"])
          : S.current.device_freezeCardHint(list[index]["name"]),
      outSideDismissible: true,
      affirmText: list[index]["freeze"]
          ? S.of(context).device_unfreeze
          : S.of(context).device_freeze,
      affirmTextColor: Colors.red,
      onPressed: (bool value) {
        if (value) _affirmFreeze(index);
      },
    );
  }

  ///确认冻结或解冻
  _affirmFreeze(int index) async {
    try {
      EasyLoading.show(status: S.current.device_loading);
      await BleDevice().freezeCard(
          widget.deviceId, list[index]["uid"], !list[index]["freeze"]);
      EasyLoading.dismiss();
      EasyLoading.showSuccess(S.current.device_success);
      initData();
    } on PlatformException catch (e) {
      print(e);
      EasyLoading.dismiss();
      WitsystemError error = AutoErrorHandle.auto(e);
      EasyLoading.showToast(error.error + "${e.message}");
    }
  }

  @override
  void dispose() {
    super.dispose();
    EasyLoading.dismiss();
  }

  @override
  Widget build(BuildContext context) {
    return FloatingActionButtonScaffold(
      title: S.of(context).device_cardManage + "($deviceName)",
      centerHint: S.of(context).device_addCard,
      floatingButtonOnTap: () {
        Navigator.of(context).push<bool>(MaterialPageRoute(builder: (context) {
          return AddCardWidget(
              deviceId: widget.deviceId, deviceName: deviceName);
        })).then((bool? value) {
          if (value != null && value) initData();
        });
      },
      crossFadeState:
          list.isEmpty ? CrossFadeState.showFirst : CrossFadeState.showSecond,
      body: RefreshIndicator(
        onRefresh: () async {
          await refurbishDevice();
          return Future.value();
        },
        child: ListView.builder(
          itemCount: list.length,
          itemBuilder: (context, index) {
            return Card(
                shape: const RoundedRectangleBorder(
                  borderRadius: BorderRadius.all(Radius.circular(15)),
                ),
                margin: const EdgeInsets.fromLTRB(10, 10, 10, 10),
                elevation: 10,
                child: InkWell(
                  borderRadius: const BorderRadius.all(Radius.circular(15)),
                  onTap: () {
                    _cardOnTap(index);
                  },
                  child: Container(
                    padding: const EdgeInsets.fromLTRB(0, 10, 0, 10),
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: [
                        ListTile(
                          leading: Icon(
                            Icons.card_giftcard,
                            size: 50,
                            color: colorList[index % 4],
                          ),
                          title: Text(list[index]["name"]),
                          subtitle: Text(list[index]["dataType"]),
                          trailing: const Icon(Icons.chevron_right),
                        ),
                        Container(
                          margin: const EdgeInsets.fromLTRB(0, 40, 20, 0),
                          alignment: Alignment.centerRight,
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.end,
                            children: [
                              Text(
                                list[index]["freeze"]
                                    ? S.of(context).device_freeze
                                    : "${S.of(context).device_normal}   ",
                                style: TextStyle(
                                    color: list[index]["freeze"]
                                        ? Colors.red
                                        : Colors.green),
                              ),
                              Text(
                                "  ${int.parse(list[index]["uid"], radix: 16).toString().toString().padLeft(4, "0")}",
                                style: const TextStyle(color: Colors.green),
                              )
                            ],
                          ),
                        ),
                        const Divider(),
                        Padding(
                          padding: const EdgeInsets.all(20),
                          child: Text(
                            list[index]["createTime"],
                            style: const TextStyle(),
                          ),
                        )
                      ],
                    ),
                  ),
                ));
          },
        ),
      ),
    );
  }
}
