import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:witsystem_core/error/entity/WitsystemError.dart';
import 'package:witsystem_core/error/handle/AutoErrorHandle.dart';
import 'package:witsystem_core/pages/sheet/WitsystemBottomSheet.dart';
import 'package:witsystem_device_manage/assets/icons/WitsIcon.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_manage/pages/home/slock/setup/Volume.dart';
import 'package:witsystem_device_plugin/code/DeviceFunctionType.dart';
import 'package:witsystem_device_plugin/command/DeviceModel.dart';
import 'package:witsystem_device_plugin/device/BleDevice.dart';
import 'package:witsystem_device_plugin/device/DeviceManage.dart';
import 'package:witsystem_device_plugin/enity/StateCode.dart';

///门锁设置
class SlockSetupWidget extends StatefulWidget {
  final String deviceId;

  const SlockSetupWidget({Key? key, required this.deviceId}) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _SlockSetupState();
  }
}

class _SlockSetupState extends State<SlockSetupWidget> {
  String deviceName = "";

  //00-正常模式 01-勿扰模式 02-常开模式 03安全模式 04-体验模式
  final Map _stateModelMap = {
    //状态码获取文字
    0: S.current.device_normalMode,
    1: S.current.device_notDisturbMode,
    2: S.current.device_normallyOpenMode,
    3: S.current.device_safeMode,
    4: S.current.device_experienceMode,
  };

  //软件编码获得文字
  final Map _softwareModelMap = {
    DeviceModel.NORMAL_MODE: S.current.device_normalMode,
    DeviceModel.NOT_DISTURB_MODE: S.current.device_notDisturbMode,
    DeviceModel.NORMALLY_OPEN_MODE: S.current.device_normallyOpenMode,
    DeviceModel.SAFE_MODE: S.current.device_safeMode,
    DeviceModel.EXPERIENCE_MODE: S.current.device_experienceMode,
  };

  StateCode? _stateCode;
  bool _isVoice = true; //是否支持语音
  bool _isCloseCard = true; //是否可以关闭刷卡

  @override
  void initState() {
    super.initState();
    _getDeviceName();
    _filter();
    _syncDeviceState();
    _filterDeviceSupportModel();
  }

  ///获得设备名称
  _getDeviceName() async {
    deviceName = (await DeviceManage().device(widget.deviceId))!["deviceName"];
    setState(() {});
  }

  ///筛选支持的功能
  _filter() async {
    _isVoice = !await DeviceManage()
        .isDeviceFunction(widget.deviceId, DeviceFunctionType.voice); //判断是否支持语音
    _isCloseCard = !await DeviceManage().isDeviceFunction(
        widget.deviceId, DeviceFunctionType.closeCard); //判断是否支持关闭卡
    _getDeviceStateCode();
  }

  ///获得设备的状态码解析信息
  _getDeviceStateCode() async {
    _stateCode = await DeviceManage()
        .getDeviceStateInfo(widget.deviceId); //获得状态码，里面有音量和卡功能是否开启和当前处于什么模式等等信息
    setState(() {});
  }

  ///筛选设备支持的模式
  _filterDeviceSupportModel() async {
    var info = await DeviceManage().getDeviceSoftwareFunction(widget.deviceId);
    _softwareModelMap
        .removeWhere((key, value) => !info!.modelType.contains(key));
  }

  ///和设备同步状态
  _syncDeviceState() async {
    EasyLoading.show(
        status: S.current.device_updateMode,
        maskType: EasyLoadingMaskType.clear);
    try {
      await BleDevice().updateDeviceState(widget.deviceId);
      EasyLoading.dismiss();
      EasyLoading.showSuccess(S.current.device_syncDeviceInfoSuccess);
      _getDeviceStateCode();
    } on PlatformException catch (e) {
      EasyLoading.dismiss();
      WitsystemError error = AutoErrorHandle.auto(e);
      EasyLoading.showToast(S
          .current
          .device_syncDeviceInfoFail(error.error + "${e.message}"));
    }
  }

  ///开启或者关闭刷卡功能
  _offOrOnCard(v) async {
    EasyLoading.show(
        status: S.current.device_execution,
        maskType: EasyLoadingMaskType.clear);
    try {
      await BleDevice().closeOrOpenCard(widget.deviceId, !v);
      EasyLoading.dismiss();
      EasyLoading.showSuccess(S.current.device_success);
      Future.delayed(const Duration(milliseconds: 360), () {
        _getDeviceStateCode();
      });
    } on PlatformException catch (e) {
      EasyLoading.dismiss();
      WitsystemError error = AutoErrorHandle.auto(e);
      EasyLoading.showToast(error.error + "${e.message}");
      _getDeviceStateCode();
    }
  }

  ///点击修改模式
  _clickUpdateModel() {
    if (_softwareModelMap.length <= 1) {
      EasyLoading.showToast(S.current.device_notUpdateMode);
      return;
    }
    showModalBottomSheet(
        context: context,
        isDismissible: true,
        barrierColor: Colors.black26,
        backgroundColor: const Color(0x00000000),
        builder: (BuildContext context) {
          return WitsystemBottomSheetWidget(
            sheetItem: _softwareModelMap.values.toList()
              ..add(S.current.device_cancel),
            onTap: (index) {
              if (index == _softwareModelMap.length) {
                return;
              }
              _updateModel(_softwareModelMap.keys.toList()[index]);
            },
          );
        });
  }

  ///修改模式
  _updateModel(DeviceModel model) async {
    EasyLoading.show(
        status: S.current.device_execution,
        maskType: EasyLoadingMaskType.clear);
    try {
      await BleDevice().setModel(widget.deviceId, model);
      EasyLoading.dismiss();
      EasyLoading.showSuccess(S.current.device_success);
      Future.delayed(const Duration(milliseconds: 360), () {
        _getDeviceStateCode();
      });
    } on PlatformException catch (e) {
      EasyLoading.dismiss();
      WitsystemError error = AutoErrorHandle.auto(e);
      EasyLoading.showToast(error.error + "${e.message}");
      _getDeviceStateCode();
    }
  }

  @override
  void dispose() {
    super.dispose();
    EasyLoading.dismiss();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
          title: Text(S.of(context).device_deviceSettings + "($deviceName)")),
      body: Container(
        margin: const EdgeInsets.fromLTRB(10, 0, 10, 10),
        child: ListView(
          children: [
            Offstage(
              offstage: _isVoice,
              child: InkWell(
                  onTap: () {
                    Navigator.of(context)
                        .push(MaterialPageRoute(builder: (context) {
                      return VolumeWidget(
                          deviceName: deviceName, deviceId: widget.deviceId);
                    })).then((value) {
                      if (value) {
                        _getDeviceStateCode();
                      }
                    });
                  },
                  child: ListTile(
                    minLeadingWidth: 10,
                    contentPadding: const EdgeInsets.all(0),
                    hoverColor: Colors.white10,
                    leading: const SizedBox(
                        width: 30,
                        height: 60,
                        child: Icon(WitsIcon.yuyin, size: 30)),
                    title: Text(S.of(context).device_lockSpeech),
                    subtitle: Text(S.of(context).device_lockSpeechHint,
                        style: const TextStyle(fontSize: 12)),
                    trailing: Row(
                      mainAxisSize: MainAxisSize.min,
                      children: [
                        Text(
                            _stateCode == null ? "" : "${_stateCode!.volume} "),
                        const Icon(WitsIcon.youjiantou, size: 20),
                      ],
                    ),
                  )),
            ),
            Offstage(
              offstage: _isVoice,
              child: const Divider(),
            ),
            Offstage(
              offstage: _isCloseCard,
              child: ListTile(
                minLeadingWidth: 10,
                contentPadding: const EdgeInsets.all(0),
                hoverColor: Colors.white10,
                leading: const SizedBox(
                    width: 30,
                    height: 60,
                    child: Icon(WitsIcon.shuaka, size: 30)),
                title: Text(S.of(context).device_swingCardFunction),
                subtitle: Text(S.of(context).device_swingCardFunctionHint,
                    style: const TextStyle(fontSize: 10)),
                trailing: Switch(
                    value: _stateCode == null ? false : _stateCode!.card == 1,
                    onChanged: _offOrOnCard),
              ),
            ),
            Offstage(
              offstage: _isCloseCard,
              child: const Divider(),
            ),
            InkWell(
                onTap: _clickUpdateModel,
                child: ListTile(
                  minLeadingWidth: 10,
                  contentPadding: const EdgeInsets.all(0),
                  hoverColor: Colors.white10,
                  leading: const SizedBox(
                      width: 30,
                      height: 60,
                      child: Icon(WitsIcon.moshi, size: 30)),
                  title: Text(S.of(context).device_updateModel),
                  subtitle: Text(S.of(context).device_updateModelHint,
                      style: const TextStyle(fontSize: 10)),
                  trailing: Row(
                    mainAxisSize: MainAxisSize.min,
                    children: [
                      Text(
                          _stateCode == null
                              ? ""
                              : _stateModelMap[_stateCode!.mode],
                          style: const TextStyle(fontSize: 12)),
                      const Icon(WitsIcon.youjiantou, size: 20)
                    ],
                  ),
                )),
            const Divider(),
          ],
        ),
      ),
    );
  }
}
